/**
 * <p>File: DifferentiableFunction.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: January 2016</p>
 * @author Faton M. Berisha
 * @version 6.0.0
 */

package numeric;

import javax.swing.*;
import javax.swing.table.*;
import java.text.*;

/**
 * An abstract table.
 */
public abstract class AbstractTable extends AbstractTableModel {

	private static final long serialVersionUID = 4;

	protected String[] columnHeader;
	protected Object[][] data;

	private static String pattern = "#.00000E0";
	private static boolean defaultPattern = false;
	private static int numSignificantDigits = 5;

	/**
	 * Initializes a table.
	 */
	public AbstractTable() {
	}

	public String getColumnName(int col) {
		return columnHeader[col];
	}

	public Object getValueAt(int row, int col) {
		return data[row][col];
	}

	/**
	 * Gets the formatting pattern for the <code>DecimalFormat</code>
	 * formatter used to render <code>Double</code> cells values.
	 * 
	 * @return the pattern
	 */
	public static String getPattern() {
		return pattern;
	}

	/**
	 * Sets the formatting pattern for the <code>DecimalFormat</code>
	 * formatter used to render <code>Double</code> cells values.
	 * 
	 * @param pattern
	 *            the pattern to set
	 */
	public static void setPattern(String pattern) {
		AbstractTable.pattern = pattern;
	}

	/**
	 * Gets the number of significant digits to display for <code>Double</code>
	 * cell values.
	 * 
	 * @return the number of significant digits
	 */
	public static int getNumSignificantDigits() {
		return numSignificantDigits;
	}

	/**
	 * Sets the number of significant digits to display for <code>Double</code>
	 * cell values.
	 * 
	 * @param numSignificantDigits
	 *            the number of significant digits to set
	 */
	public static void setNumSignificantDigits(int numSignificantDigits) {
		if (numSignificantDigits > 0) {
			AbstractTable.numSignificantDigits = numSignificantDigits;
		} else {
			throw new IllegalArgumentException(
					"Illegal number of significant digits"
							+ numSignificantDigits);
		}
	}

	/**
	 * Gets if there is a default formatting pattern for the
	 * <code>DecimalFormat</code> formatter used to render <code>Double</code>
	 * cells values.
	 * 
	 * @return the default pattern
	 */
	public static boolean isDefaultPattern() {
		return defaultPattern;
	}

	/**
	 * Sets the default formatting pattern for the <code>DecimalFormat</code>
	 * formatter used to render <code>Double</code> cells values.
	 * 
	 * @param default
	 *            pattern the defaultPattern to set
	 */
	public static void setDefaultPattern(boolean defaultPattern) {
		AbstractTable.defaultPattern = defaultPattern;
	}

	protected static class DoubleRenderer extends DefaultTableCellRenderer {

		private static final long serialVersionUID = 3;

		private NumberFormat formatter;

		public void setValue(Object value) {
			if (formatter == null) {
				formatter = NumberFormat.getInstance();
			}
			if (!isDefaultPattern()) {
				setPattern(getSignificantDigitsPattern(
						getNumSignificantDigits(), value));
			}
			if (formatter instanceof DecimalFormat) {
				((DecimalFormat) formatter).applyPattern(pattern);
			}
			setHorizontalAlignment(SwingConstants.RIGHT);
			setText(value == null ? "" : formatter.format(value));
		}

		private String getSignificantDigitsPattern(int numSignificantDigits,
				Object value) {
			StringBuffer sb = new StringBuffer();
			double doubleVal = 0;
			try {
				if (value != null) {
					doubleVal = ((Double) value).doubleValue();
				}
			} catch (NumberFormatException e) {
				System.err.println(e.getMessage());
			}
			int exponent = 0;
			if (doubleVal != 0.0) {
				exponent = (int) Math.floor(Math.log10(Math.abs(doubleVal))) + 1;
			}
			int numDigits = Math.min(exponent, numSignificantDigits);
			if (numDigits < 0) {
				sb.append("'0'#");
				numDigits = 0;
			} else {
				sb.append('0');
				for (int i = 1; i < numDigits; i++) {
					sb.append('0');
				}
			}
			if (numDigits < numSignificantDigits) {
				sb.append('.');
				for (int i = numDigits; i < numSignificantDigits; i++) {
					sb.append('0');
				}
			}
			if (exponent < 0 || exponent > numSignificantDigits) {
				sb.append("E0");
			}
			return sb.toString();
		}
	}

}
